%{
/*
   This file is part of the index package,
   Copyright (C) 1997 Harold Tay, haroldtay@pacific.net.sg

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA
 */
/* Given the pathname to an html document, and several patterns,
   the first line of output is the title of that document, the
   next lines are the context in which each pattern is found within
   that document.
 */
#include <regex.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <ctype.h>

#define STACKSIZE 2
#define MAXMATCH 2

void puttitle(void);
void checkword(void);
int done_put = 0;
regex_t * re;
int * counts;
char ** patterns;
int nr_patterns;
%}

spc [ \t\r\n\f]
notclose ([^>]|\n)
title "<"{spc}*title{notclose}*">"{notclose}*"<"{spc}*"/"title{notclose}*">"
comment "<!--"(.|\n)*"-->"
tag "<"{notclose}*">"
word [^ \t\r\n<>]{1,15}
chr &(#[0123456789]{1,4})|([abcdefghijklmnopqrstuvwxyz]{2,3});

%%

{title}            {
                     puttitle();
                   }
{tag}              ;
{chr}              ;
{comment}          ;
{word}             {
                     checkword();
                   }
.|\n               ;

%%


void
puttitle(void)
{
  char * start;
  char * end;
  char * cp;

  if( done_put )return;
  if( NULL == (start = strchr(yytext, '>')) )return;
  start++;
  if( NULL == (end = strchr(start, '<')) )return;
  *end = '\0';
  for(cp = start; *cp; cp++)
    if( '\n' == *cp )*cp = ' ';
  printf("%s\n", start);
  done_put = 1;
}

static char * stack[STACKSIZE] = {0,};

void
push(void)
{
  int i;
  if( stack[0] )free(stack[0]);
  for(i = 1; i < STACKSIZE; i++)
    stack[i-1] = stack[i];
  stack[STACKSIZE - 1] = strdup(yytext);
}

static int
string_casestr(char * hay, char * needle)
{
  while( *hay && *needle != tolower(*hay) )hay++;
  if( *hay ){
    while( *needle == tolower(*hay) ){ hay++; needle++; }
    if( *needle )return(0);
    else return(1);
  }else
    return(0);
}

void
checkword(void)
{
  static int more = 0;
  int i;

  push();

  if( more ){
    printf("%s", yytext);
    if( 0 == --more )printf("\n");
    else printf(" ");
    return;
  }

  for(i = 0; i < nr_patterns; i++){
    int match;
    if( counts[i] >= MAXMATCH )continue;
    if( re )
      match = (0 == regexec(&re[i], yytext, 0, NULL, 0));
    else
      match = (string_casestr(yytext, patterns[i]));
    if( match ){
      counts[i]++;
      if( ! done_put ){ done_put = 1; printf("\n"); }
      for(i = 0; i < STACKSIZE; i++)
        if( stack[i] )
          printf("%s ", stack[i]);
      more = STACKSIZE;
    }
  }
}

int yywrap(void) { return(1); }

int
main(int argc, char ** argv)
{
  int i;

  if( 0 == strcmp(argv[1], "-r") ){
    re = malloc(nr_patterns * sizeof(regex_t));
    patterns = &argv[3];
    yyin = fopen(argv[2], "r");
    nr_patterns = argc - 3;
  }else{
    re = NULL;
    patterns = &argv[2];
    yyin = fopen(argv[1], "r");
    nr_patterns = argc - 2;
  }

  if( NULL == yyin ){
    fprintf(stderr, "Can't open %s: %s\n", argv[1], strerror(errno));
    exit(1);
  }

  counts = calloc(nr_patterns, sizeof(int));

  if( re )
    for(i = 0; i < nr_patterns; i++)
      if( regcomp(&re[i], argv[i + 2], REG_EXTENDED|REG_NOSUB) ){
        fprintf(stderr, "Can't compile %s\n", argv[i + 2]);
        exit(2);
      }

  yyrestart(yyin);
  yylex();
  free(counts);
  if( re ){
    for(i = 0; i < nr_patterns; i++) regfree(&re[i]);
    free(re);
  }
  for(i = 0; i < STACKSIZE; i++)
    if( stack[i] )free(stack[i]);
  exit(0);
}

