/*
   This file is part of the index package,
   Copyright (C) 1997 Harold Tay, haroldtay@pacific.net.sg

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA
 */
/* string allocation routines.  Everything is allocated from one
   list of arenas
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define ARENASIZE 4096

struct list{
  char * arena;
  int size;
  int used;
  struct list * next;
};

static struct list * list = NULL;

int
string_init(void)
{
  if( NULL == list )
    if( NULL == (list = calloc(1, sizeof(struct list))) )
      return(-1);
  return(0);
}

char *
string_dup(char * string)
{
  char * tmp;
  int len = strlen(string) + 1;

  if( NULL == list->arena ){
    list->arena = malloc(ARENASIZE);
    if( NULL == list->arena )
      return(NULL);
    list->size = ARENASIZE;
    list->used = 0;
  }

  if( len >= list->size )
    return(NULL);  /* too big */

  if( list->used + len > list->size ){
    struct list * newlist = calloc(1, sizeof(struct list));
    if( NULL == newlist )return(NULL);
    newlist->arena = malloc(ARENASIZE);
    if( NULL == newlist->arena ){
      free(newlist); return(NULL); }
    newlist->size = ARENASIZE;
    newlist->used = 0;
    newlist->next = list;
    list = newlist;
  }

  tmp = list->arena + list->used;
  strcpy(tmp, string);
  list->used += len;
  return(tmp);
}

void
string_free(void)
{
  struct list * lp;
  struct list * next;

  for(lp = list; lp; lp = next){
    if( lp->arena )
      free(lp->arena);
    next = lp->next;
    free(lp);
  }
  list = NULL;
}

